import json
import argparse
import numpy as np
import base64
import cv2

parser = argparse.ArgumentParser(description='Finding templates on an image')
parser.add_argument('image', type=str, help='path to image file .jpg or .png')
parser.add_argument('temp', type=str, help='base64 fragment file')
args = parser.parse_args()


def edge(img):
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    edged = cv2.Canny(gray, 30, 200)
    contours, hierarchy = cv2.findContours(edged,
                                           cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
    return edged


def resize(img):
    scale_percent = 400  # percent of original size
    width = int(img.shape[1] * scale_percent / 100)
    height = int(img.shape[0] * scale_percent / 100)
    dim = (width, height)
    resized = cv2.resize(img, dim, interpolation=cv2.INTER_AREA)
    return resized


def increase_brightness(img, value=100):
    hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
    h, s, v = cv2.split(hsv)

    lim = 255 - value
    v[v > lim] = 255
    v[v <= lim] += value

    final_hsv = cv2.merge((h, s, v))
    img = cv2.cvtColor(final_hsv, cv2.COLOR_HSV2BGR)
    return img


def check_similarity(img, temp):
    img = edge(img)
    temp = edge(temp)

    h1, w1 = img.shape
    h2, w2 = temp.shape

    q11 = img[0:int(h1 * 0.5), 0:int(w1 * 0.5)]
    q12 = img[0:int(h1 * 0.5), int(w1 * 0.5):w1]
    q13 = img[int(h1 * 0.5):h1, 0:int(w1 * 0.5)]
    q14 = img[int(h1 * 0.5):h1, int(w1 * 0.5):w1]

    q21 = temp[0:int(h2 * 0.5), 0:int(w2 * 0.5)]
    q22 = temp[0:int(h2 * 0.5), int(w2 * 0.5):w2]
    q23 = temp[int(h2 * 0.5):h2, 0:int(w2 * 0.5)]
    q24 = temp[int(h2 * 0.5):h2, int(w2 * 0.5):w2]

    coeff = 3
    if h1 * w1 < 800:
        coeff = 5

    wq1d = abs(np.sum(q11 == 255) * coeff - np.sum(q21 == 255) * coeff)
    wq2d = abs(np.sum(q12 == 255) * coeff - np.sum(q22 == 255) * coeff)
    wq3d = abs(np.sum(q13 == 255) * coeff - np.sum(q23 == 255) * coeff)
    wq4d = abs(np.sum(q14 == 255) * coeff - np.sum(q24 == 255) * coeff)

    return max(wq4d, wq3d, wq2d, wq1d)


def check_colors(image1, image2):
    hist1 = cv2.calcHist(image1, [0, 1, 2], None, [8, 8, 8],
                         [0, 256, 0, 256, 0, 256])

    hist2 = cv2.calcHist(image2, [0, 1, 2], None, [8, 8, 8],
                         [0, 256, 0, 256, 0, 256])
    hist1 = cv2.normalize(hist1, hist1).flatten()
    hist2 = cv2.normalize(hist2, hist2).flatten()
    d = cv2.compareHist(hist1, hist2, cv2.HISTCMP_CORREL)
    return d


# <-------------------------Read Images---------------------------->
with open(args.temp, "r") as f:
    text = f.read()
im_bytes = base64.b64decode(text)
im_arr = np.frombuffer(im_bytes, dtype=np.uint8)  # im_arr is one-dim Numpy array
template = cv2.imdecode(im_arr, flags=cv2.IMREAD_COLOR)
image = cv2.imread(args.image)
# <-------------------------Read Images---------------------------->

# <-------------------------Thresholds----------------------------->
contour_similarity_threshold = 35  # Разница картинок по площади контуров по квадрантам.
# Чем выше тем больше различие изображений. Исходя из тестов у похожих картинок он не превышает 150

color_similarity_threshold = 0.1  # Порог расхождения цветовой гистограммы по методу HISTCMP_CORREL. 1 для абсолютно
# идентичных изображений и < 0.1 для абсолютно разных.
# <-------------------------Thresholds----------------------------->


# <-------------------------Processing----------------------------->
image = increase_brightness(image)
template = increase_brightness(template)
heat_map = cv2.matchTemplate(image, template, cv2.TM_CCOEFF_NORMED)
min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(heat_map)
h, w, _ = template.shape
y, x = np.unravel_index(np.argmax(heat_map), heat_map.shape)
new_image = image[y + 4:y + h - 4, x + 4:x + w - 4]
new_template = template[4:h - 4, 4:w - 4]
# <-------------------------Processing----------------------------->
# <-------------------------conf----------------------------->
#y = y - 1
#x = x - 12
#if y < 0:
#    y = 0
#if x < 0:
#    x = 0
# <-------------------------conf----------------------------->
# <-------------------------Results----------------------------->
contour_similarity_index = check_similarity(new_image, new_template)
color_similarity_index = check_colors(new_image, new_template)


#contourMatch = True
#colorMatch = True
#if contour_similarity_index > contour_similarity_threshold:
#    contourMatch = False

#if color_similarity_index < color_similarity_threshold:
#    colorMatch = False

# <-------------------------Results----------------------------->

# <-------------------------Output----------------------------->
output = {"xpos": int(x),
          "ypos": int(y),
          "height": int(h),
          "width": int(w),
          #"cntM": contourMatch,
          #"clrM": colorMatch,
          "contour_similarity_index": int(contour_similarity_index),
          "color_similarity_index": float(color_similarity_index)
          }
json_dump = json.dumps(output)
print(json_dump)

